import java.io.*;
import java.security.*;
import java.util.*;
import javax.net.ssl.*;

import org.apache.http.*;
import org.apache.http.conn.*;
import org.apache.http.conn.ssl.*;
import org.apache.http.client.methods.*;
import org.apache.http.impl.client.*;

public class Client {
    public final static String db_password = "nss.SECret.123";

    public final static String[] protocols = {
        "TLSv1.2",
    };

    public final static String[] cipher_suites = {
        /* List from:
         * https://bugzilla.redhat.com/show_bug.cgi?id=1760838#c14
         */
        "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_RSA_WITH_AES_128_CBC_SHA256",
        "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256",
        "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
        "TLS_DHE_RSA_WITH_AES_256_CBC_SHA256",
        "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384",
        "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
        "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
        "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
        "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
        "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",
        "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
        "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
        "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
        "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
        "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
        "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
        "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384"
    };

    public static KeyStore ks;
    public static KeyManagerFactory kmf;
    public static TrustManagerFactory tmf;
    public static SecureRandom sr;

    public static void init() throws Exception {
        ks = KeyStore.getInstance("PKCS11", "SunPKCS11-NSS-FIPS");
        kmf = KeyManagerFactory.getInstance("SunX509", "SunJSSE");
        tmf = TrustManagerFactory.getInstance("SunX509", "SunJSSE");
        sr = SecureRandom.getInstance("PKCS11", "SunPKCS11-NSS-FIPS");

        ks.load(null, db_password.toCharArray());
        kmf.init(ks, db_password.toCharArray());
        tmf.init(ks);
        sr.nextBytes(new byte[5]);
    }

    public static void testHandshake(String protocol, String host, String port) throws Exception {
        System.out.println("Testing Cipher Suites under protocol: " + protocol);

        for (String cipher_suite : cipher_suites) {
            System.out.println("=====START " + cipher_suite + "=====");
            try {
                SSLContext ctx = SSLContext.getInstance(protocol, "SunJSSE");
                KeyManager[] kms = kmf.getKeyManagers();
                TrustManager[] tms = tmf.getTrustManagers();

                System.out.println("Number of KMs: " + kms.length);
                System.out.println("Number of TMs: " + tms.length);

                ctx.init(kms, tms, sr);

                /* Largely from
                 * https://stackoverflow.com/questions/5206010/using-apache-httpclient-for-https
                 */

                SSLConnectionSocketFactory scsf = new SSLConnectionSocketFactory(ctx,
                    protocols,
                    new String[] { cipher_suite },
                    SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER
                );

                CloseableHttpClient chc = HttpClients.custom().setHostnameVerifier(SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER).setSSLSocketFactory(scsf).build();

                try {
                    HttpGet httpget = new HttpGet("https://" + host + ":" + port + "/");

                    CloseableHttpResponse response = chc.execute(httpget);
                    try {
                        HttpEntity entity = response.getEntity();
                        response.getStatusLine();

                        System.out.println(" - " + cipher_suite + " :: OK");
                    } finally {
                        response.close();
                    }
                } finally {
                    chc.close();
                }
            } catch (Exception e) {
                System.out.println(" - " + cipher_suite + " :: Failed");
                System.err.println(e.getMessage());
                e.printStackTrace(System.err);
            }
        }
    }

    public static void main(String[] args) throws Exception {
        init();

        String protocol = "TLS";
        String host = "google.com";
        String port = "443";

        if (args.length >= 1) {
            protocol = args[0];
        }
        if (args.length >= 2) {
            host = args[1];
        }
        if (args.length >= 3) {
            port = args[2];
        }

        testHandshake(protocol, host, port);
    }
}

